
/* Generated data (by glib-mkenums) */

#ifndef WEBKIT_ENUM_TYPES_H
#define WEBKIT_ENUM_TYPES_H

#include <glib-object.h>

#include <webkit/webkitdefines.h>

G_BEGIN_DECLS

#include <webkit/webkitwebnavigationaction.h>

#define WEBKIT_TYPE_WEB_NAVIGATION_REASON webkit_web_navigation_reason_get_type()

WEBKIT_API GType
webkit_web_navigation_reason_get_type(void);

#include <webkit/webkitwebview.h>

#define WEBKIT_TYPE_NAVIGATION_RESPONSE webkit_navigation_response_get_type()

WEBKIT_API GType
webkit_navigation_response_get_type(void);

#define WEBKIT_TYPE_WEB_VIEW_TARGET_INFO webkit_web_view_target_info_get_type()

WEBKIT_API GType
webkit_web_view_target_info_get_type(void);

G_END_DECLS

#endif

/* Generated data ends here */

